/****
HidTouch input driver.
(c)2008 David SPORN
 ****/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef HIDTOUCH__HDTCONFIGURATION__H
#define HIDTOUCH__HDTCONFIGURATION__H

static void
HdtConfiguration__retrieve(HdtConfigurationPtr configuration, IDevPtr dev)
{
    /* Device */
    configuration->device = xf86CheckStrOption(dev->commonOptions,
            HDT__CONFIGURATION_NAME__DEVICE, HDT__CONFIGURATION_DEFAULT__DEVICE);
    xf86Msg(X_INFO, "%s: Using device %s.\n", dev->identifier, configuration->device);

    /* Packet count */
    configuration->packetCount = xf86CheckIntOption(dev->commonOptions,
            HDT__CONFIGURATION_NAME__PACKET_COUNT, HDT__CONFIGURATION_DEFAULT__PACKET_COUNT);
    xf86Msg(X_INFO, "%s: Reading %i packets.\n", dev->identifier, configuration->packetCount);

    /* OpCodes */
    configuration->opCodeSetup.pressure = xf86CheckIntOption(dev->commonOptions,
            HDT__CONFIGURATION_NAME__OPCODE_PRESSURE, HDT__CONFIGURATION_DEFAULT__OPCODE_PRESSURE);
    xf86Msg(X_INFO, "%s: Watching %x for pressure data.\n", dev->identifier, configuration->opCodeSetup.pressure);
    configuration->opCodeSetup.x = xf86CheckIntOption(dev->commonOptions,
            HDT__CONFIGURATION_NAME__OPCODE_X, HDT__CONFIGURATION_DEFAULT__OPCODE_X);
    xf86Msg(X_INFO, "%s: Watching %x for x data.\n", dev->identifier, configuration->opCodeSetup.x);
    configuration->opCodeSetup.y = xf86CheckIntOption(dev->commonOptions,
            HDT__CONFIGURATION_NAME__OPCODE_Y, HDT__CONFIGURATION_DEFAULT__OPCODE_Y);
    xf86Msg(X_INFO, "%s: Watching %x for y data.\n", dev->identifier, configuration->opCodeSetup.y);

    /* Calibration */
    configuration->calibration.model = xf86CheckIntOption(dev->commonOptions,
    		HDT__CONFIGURATION_NAME__CALIBRATION__MODEL, HDT__CONFIGURATION_DEFAULT__CALIBRATION__MODEL);
    if (HDT__CONFIGURATION_VALUE__CALIBRATION__MODEL__FOUR_CORNER == configuration->calibration.model)
    {
        xf86Msg(X_INFO, "%s: CalibrationModel : %s = %i (FourCorner).\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__MODEL, configuration->calibration.model);


        configuration->calibration.screenWidth = xf86CheckIntOption(dev->commonOptions,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER__SCREEN_WIDTH, HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER__SCREEN_WIDTH);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER__SCREEN_WIDTH, configuration->calibration.screenWidth);
        configuration->calibration.screenHeight = xf86CheckIntOption(dev->commonOptions,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER__SCREEN_HEIGHT, HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER__SCREEN_HEIGHT);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER__SCREEN_HEIGHT, configuration->calibration.screenHeight);


        configuration->calibration.cornerTopLeftX = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_LEFT_X, HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_TOP_LEFT_X);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_LEFT_X, configuration->calibration.cornerTopLeftX);
        configuration->calibration.cornerTopLeftY = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_LEFT_Y, HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_TOP_LEFT_Y);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_LEFT_Y, configuration->calibration.cornerTopLeftY);
        configuration->calibration.cornerTopRightX = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_RIGHT_X, HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_TOP_RIGHT_X);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_RIGHT_X, configuration->calibration.cornerTopRightX);
        configuration->calibration.cornerTopRightY = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_RIGHT_Y, HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_TOP_RIGHT_Y);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_TOP_RIGHT_Y, configuration->calibration.cornerTopRightY);

        configuration->calibration.cornerBottomLeftX = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_LEFT_X, HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_BOTTOM_LEFT_X);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_LEFT_X, configuration->calibration.cornerBottomLeftX);
        configuration->calibration.cornerBottomLeftY = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_LEFT_Y, HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_BOTTOM_LEFT_Y);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_LEFT_Y, configuration->calibration.cornerBottomLeftY);
        configuration->calibration.cornerBottomRightX = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_RIGHT_X, HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_BOTTOM_RIGHT_X);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_RIGHT_X, configuration->calibration.cornerBottomRightX);
        configuration->calibration.cornerBottomRightY = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_RIGHT_Y, HDT__CONFIGURATION_DEFAULT__CALIBRATION__CORNER_BOTTOM_RIGHT_Y);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__CORNER_BOTTOM_RIGHT_Y, configuration->calibration.cornerBottomRightY);
    }
    else
    {
        xf86Msg(X_INFO, "%s: CalibrationModel : %s = %i (MinMax).\n", dev->identifier,
        		HDT__CONFIGURATION_NAME__CALIBRATION__MODEL, configuration->calibration.model);
        /* force the value to 0 */
        configuration->calibration.model = HDT__CONFIGURATION_VALUE__CALIBRATION__MODEL__MIN_MAX ;


        configuration->calibration.minX = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__MIN_X, HDT__CONFIGURATION_DEFAULT__CALIBRATION__MIN_X);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
                HDT__CONFIGURATION_NAME__CALIBRATION__MIN_X, configuration->calibration.minX);
        configuration->calibration.minY = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__MIN_Y, HDT__CONFIGURATION_DEFAULT__CALIBRATION__MIN_Y);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
                HDT__CONFIGURATION_NAME__CALIBRATION__MIN_Y, configuration->calibration.minY);
        configuration->calibration.maxX = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__MAX_X, HDT__CONFIGURATION_DEFAULT__CALIBRATION__MAX_X);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
                HDT__CONFIGURATION_NAME__CALIBRATION__MAX_X, configuration->calibration.maxX);
        configuration->calibration.maxY = xf86CheckIntOption(dev->commonOptions,
                HDT__CONFIGURATION_NAME__CALIBRATION__MAX_Y, HDT__CONFIGURATION_DEFAULT__CALIBRATION__MAX_Y);
        xf86Msg(X_INFO, "%s: Calibrating : %s = %i.\n", dev->identifier,
                HDT__CONFIGURATION_NAME__CALIBRATION__MAX_Y, configuration->calibration.maxY);
    }
}


#endif /** ifndef HIDTOUCH__HDTCONFIGURATION__H **/
