/****
HidTouch input driver.
(c)2008-2009 David SPORN
 ****/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef HIDTOUCH__FISHNETLOCATOR__H
#define HIDTOUCH__FISHNETLOCATOR__H


void HdtFishNetLocator__destroy(HdtFishNetLocatorPtr fishnet)
{
	if (fishnet->horizontalNet)
	{
		xfree(fishnet->horizontalNet) ;
		fishnet->horizontalNet = 0 ;
	}
	if (fishnet->verticalNet)
	{
		xfree(fishnet->verticalNet) ;
		fishnet->verticalNet = 0 ;
	}
}

int HdtFishNetLocator__instanciate(HdtFishNetLocatorPtr fishnet, int width, int height)
{
	fishnet->horizontalNet = xcalloc(1, sizeof(fp8)*3*width);
	fishnet->verticalNet = xcalloc(1, sizeof(fp8)*3*height);
	if (!(fishnet->horizontalNet) || !(fishnet->verticalNet))
	{
		HdtFishNetLocator__destroy(fishnet) ;
		return 0 ;
	}
	fishnet->width = width ;
	fishnet->height = height ;
	return 1 ;
}

void HdtFishNetLocator__initGrid(fp8 grid[], int type, int size, int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
{
	//P1 to P2, P3 to P4
	int index = 0 ;
	int i=0 ;
    for(i=0 ; i < size; i++)
    {
        fp8 px1 = FixedPoint8__interpolate(x1, x2, size, i);
        fp8 py1 = FixedPoint8__interpolate(y1, y2, size, i);
        fp8 px2 = FixedPoint8__interpolate(x3, x4, size, i);
        fp8 py2 = FixedPoint8__interpolate(y3, y4, size, i);

        if (0 == type)
        {
            grid[index + 0] = -FixedPoint8__sub(py2, py1) ;
            grid[index + 1] = -FixedPoint8__sub(px1, px2) ;
        }
        else
        {
            grid[index + 0] = -FixedPoint8__sub(py1, py2) ;
            grid[index + 1] = -FixedPoint8__sub(px2, px1) ;
        }
        grid[index + 2] = -FixedPoint8__add(FixedPoint8__mul(grid[index + 0], px1),FixedPoint8__mul(grid[index + 1], py1));
        /*
        xf86Msg(X_INFO,
        		"initGrid;%i;%i;%i;%i;;%i;%i;%i;;%i;%i;%i;;%i;%i;%i;;%i;%i;%i;\n"
        		,i
        		,x1, x2, px1
        		,y1, y2, py1
        		,x3, x4, px2
        		,y3, y4, py2
        		,grid[index + 0], grid[index + 1], grid[index + 2]
        		);
		*/
        index += 3 ;

    }
}

void HdtFishNetLocator__init(HdtFishNetLocatorPtr fishnet, int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
{
	/*
	 * P1--------P2
	 *  |        |
	 *  |        |
	 * P3--------P4
	 */
	//P1 to P2, P3 to P4
	HdtFishNetLocator__initGrid(fishnet->horizontalNet, 0, fishnet->width
			,x1 ,y1
			,x2 ,y2
			,x3 ,y3
			,x4 ,y4
			);
	//P1 to P3, P2 to P4
	HdtFishNetLocator__initGrid(fishnet->verticalNet, 1, fishnet->height
			,x1 ,y1
			,x3 ,y3
			,x2 ,y2
			,x4 ,y4
			);
	//scaling factors
	int _min_x = (x1 < x3) ? x1 : x3 ;
	int _max_x = (x2 < x4) ? x4 : x2 ;
	int _min_y = (y1 < y2) ? y1 : y2 ;
	int _max_y = (y3 < y4) ? y4 : y3 ;
	fishnet->rawToGridFactors[0][0] = FixedPoint8__new(_min_x);
	fishnet->rawToGridFactors[0][1] = FixedPoint8__div(FixedPoint8__new(fishnet->width), FixedPoint8__sub(FixedPoint8__new(_max_x),fishnet->rawToGridFactors[0][0]));
	fishnet->rawToGridFactors[1][0] = FixedPoint8__new(_min_y);
	fishnet->rawToGridFactors[1][1] = FixedPoint8__div(FixedPoint8__new(fishnet->height), FixedPoint8__sub(FixedPoint8__new(_max_y),fishnet->rawToGridFactors[1][0]));
}

/**
 * Grant that a candidate is within bounds.
 *
 * @param fishnet the HdtFishNetLocator to use
 * @param candidate the canditate to cap
 * @param axis 0 = x axis, 1 = y axis
 * @return the correct value
 **/
int HdtFishNetLocator__capCandidate(HdtFishNetLocatorPtr fishnet, int candidate, int axis)
{
	int _candidate = candidate ;
	if (0 > _candidate)
	{
		_candidate = 0 ;
	}
	else
	{
		int _max = (0 == axis) ? fishnet->width : fishnet->height ;
		if (_candidate >= _max)
		{
			_candidate = _max - 1 ;
		}
	}
	return _candidate ;
}

/**
 * Compute roughly a candidate from raw coordinates along an axis.
 *
 * @param fishnet the HdtFishNetLocator to use
 * @param candidate the canditate to cap
 * @param axis 0 = x axis, 1 = y axis
 * @return a candidate coordinate
 **/
int HdtFishNetLocator__findFirstCandidate(HdtFishNetLocatorPtr fishnet, fp8 x, int axis)
{
	fp8* _coefs = fishnet->rawToGridFactors[axis] ;
	fp8 _result = FixedPoint8__sub(x, _coefs[0]) ;
	_result = FixedPoint8__mul(_coefs[1], _result) ;
	int _intResult = FixedPoint8__toInt(_result) ;
	return _intResult ; /*HdtFishNetLocator__capCandidate(fishnet, _intResult, axis) ;*/
}

//==== Horizontal processing ====

/**
 * Compute roughly a candidate from raw coordinates along the x axis.
 *
 * @param fishnet the HdtFishNetLocator to use
 * @param candidate the canditate to cap
 * @return a candidate coordinate
 **/
int HdtFishNetLocator__locateHorizontalPosition__findFirstCandidate(HdtFishNetLocatorPtr fishnet, fp8 x)
{
	return HdtFishNetLocator__findFirstCandidate(fishnet, x, 0) ;
}

/**
 * Grant that a candidate is within bounds.
 *
 * @param fishnet the HdtFishNetLocator to use
 * @param candidate the canditate to cap
 * @return the correct value
 **/
int HdtFishNetLocator__locateHorizontalPosition__capCandidate(HdtFishNetLocatorPtr fishnet, int candidate)
{
	return HdtFishNetLocator__capCandidate(fishnet, candidate, 0) ;
}

/**
 * Test that a point is above a candidate line.
 *
 * A point is valid if it lies in the positive region defined by the
 * equation of the line.
 *
 * @param fishnet the HdtFishNetLocator to use
 * @param x the point to test
 * @param y the point to test
 * @param candidate the canditate to cap
 * @return 0 = not valid, 1 = valid
 **/
int HdtFishNetLocator__locateHorizontalPosition__isValidCandidate(HdtFishNetLocatorPtr fishnet, fp8 x, fp8 y, int candidate)
{
	int _capped_candidate = HdtFishNetLocator__locateHorizontalPosition__capCandidate(fishnet, candidate) ;
	fp8* _coefs = &(fishnet->horizontalNet[_capped_candidate*3]) ;
	fp8 _a_mul_x = FixedPoint8__mul(_coefs[0], x) ;
	fp8 _b_mul_y = FixedPoint8__mul(_coefs[1], y) ;
	fp8 _result = FixedPoint8__add(_a_mul_x, _b_mul_y) ;
	_result = FixedPoint8__add(_coefs[2], _result) ;
	/*
    xf86Msg(X_INFO, "isValidCandidate : a*x + b*y + c >= 0\n a*x = %i * %i = %i\n b*y = %i * %i = %i\n c   = %i \n ====> %i .\n"
    		,_coefs[0], x, _a_mul_x
    		,_coefs[1], y, _b_mul_y
    		,_coefs[2]
    		,_result
    		);
    xf86Msg(X_INFO, "isValidCandidate = %i .\n",
    		((_result >= 0) ? 1 : 0));
    */

	return (_result >= 0) ? 1 : 0 ;
}

/**
 * Compute recursively the horizontal coordinate.
 *
 * @param fishnet the HdtFishNetLocator to use
 * @param x the point to test
 * @param y the point to test
 * @param candidate the highest canditate
 * @param dx the next canditate might be candidate - dx
 * @return the final candidate.
 **/
int HdtFishNetLocator__locateHorizontalPosition__goNextStep(HdtFishNetLocatorPtr fishnet, fp8 x, fp8 y, int candidate, int dx)
{
	if (0 == dx)
	{
		return candidate ;
	}
	int _new_dx = dx / 2 ;
	/*
    xf86Msg(X_INFO, "locateHorizontalPosition : dx %i : candidate %i.\n",
    		dx, candidate);
    */
	if (1 == HdtFishNetLocator__locateHorizontalPosition__isValidCandidate(fishnet, x, y, candidate))
	{
		int _new_candidate = candidate - dx ;
		/*
	    xf86Msg(X_INFO, "locateHorizontalPosition : _new_candidate %i.\n",
	    		_new_candidate);
	    */
		if (1 == HdtFishNetLocator__locateHorizontalPosition__isValidCandidate(fishnet, x, y, _new_candidate))
		{
			return (HdtFishNetLocator__locateHorizontalPosition__goNextStep(fishnet, x, y, _new_candidate, _new_dx)) ;
		}
		else
		{
			return (HdtFishNetLocator__locateHorizontalPosition__goNextStep(fishnet, x, y, candidate, _new_dx)) ;
		}
	}
	else
	{
		int _new_candidate = candidate + dx ;
		/*
	    xf86Msg(X_INFO, "locateHorizontalPosition : _new_candidate %i.\n",
	    		_new_candidate);
	    */
		return (HdtFishNetLocator__locateHorizontalPosition__goNextStep(fishnet, x, y, _new_candidate, _new_dx)) ;
	}
}

/**
 * Compute the horizontal coordinate.
 *
 * @param fishnet the HdtFishNetLocator to use
 * @param x the point to test
 * @param y the point to test
 * @param step the number of step, at most.
 * @return the final candidate.
 **/
int HdtFishNetLocator__locateHorizontalPosition(HdtFishNetLocatorPtr fishnet, fp8 x, fp8 y, int step)
{
	int _candidate = HdtFishNetLocator__locateHorizontalPosition__findFirstCandidate(fishnet, x) ;
	int _dx = 1 << step ;
	return HdtFishNetLocator__locateHorizontalPosition__capCandidate(
			fishnet, HdtFishNetLocator__locateHorizontalPosition__goNextStep(fishnet, x, y, _candidate, _dx)) ;

}



//==== Vertical processing ====


int HdtFishNetLocator__locateVerticalPosition__findFirstCandidate(HdtFishNetLocatorPtr fishnet, fp8 x)
{
	return HdtFishNetLocator__findFirstCandidate(fishnet, x, 1) ;
}

int HdtFishNetLocator__locateVerticalPosition__capCandidate(HdtFishNetLocatorPtr fishnet, int candidate)
{
	return HdtFishNetLocator__capCandidate(fishnet, candidate, 1) ;
}

int HdtFishNetLocator__locateVerticalPosition__isValidCandidate(HdtFishNetLocatorPtr fishnet, fp8 x, fp8 y, int candidate)
{
	int _capped_candidate = HdtFishNetLocator__locateVerticalPosition__capCandidate(fishnet, candidate) ;
	fp8* _coefs = &(fishnet->verticalNet[_capped_candidate*3]) ;
	fp8 _a_mul_x = FixedPoint8__mul(_coefs[0], x) ;
	fp8 _b_mul_y = FixedPoint8__mul(_coefs[1], y) ;
	fp8 _result = FixedPoint8__add(_a_mul_x, _b_mul_y) ;
	_result = FixedPoint8__add(_coefs[2], _result) ;
	return (_result > 0) ? 1 : 0 ;
}

int HdtFishNetLocator__locateVerticalPosition__goNextStep(HdtFishNetLocatorPtr fishnet, fp8 x, fp8 y, int candidate, int dy)
{
	if (0 == dy)
	{
		return candidate ;
	}
	int _new_dy = dy / 2 ;
	if (1 == HdtFishNetLocator__locateVerticalPosition__isValidCandidate(fishnet, x, y, candidate))
	{
		int _new_candidate = candidate - dy ;
		if (1 == HdtFishNetLocator__locateVerticalPosition__isValidCandidate(fishnet, x, y, _new_candidate))
		{
			return (HdtFishNetLocator__locateVerticalPosition__goNextStep(fishnet, x, y, _new_candidate, _new_dy)) ;
		}
		else
		{
			return (HdtFishNetLocator__locateVerticalPosition__goNextStep(fishnet, x, y, candidate, _new_dy)) ;
		}
		/*return (HdtFishNetLocator__locateVerticalPosition__goNextStep(fishnet, x, y, _new_candidate, _new_dy)) ;*/
	}
	else
	{
		int _new_candidate = candidate + dy ;
		return (HdtFishNetLocator__locateVerticalPosition__goNextStep(fishnet, x, y, _new_candidate, _new_dy)) ;
	}
}

int HdtFishNetLocator__locateVerticalPosition(HdtFishNetLocatorPtr fishnet, fp8 x, fp8 y, int step)
{
	int _candidate = HdtFishNetLocator__locateVerticalPosition__findFirstCandidate(fishnet, y) ;
	int _dy = 1 << step ;
	return HdtFishNetLocator__locateVerticalPosition__capCandidate(
			fishnet, HdtFishNetLocator__locateVerticalPosition__goNextStep(fishnet, x, y, _candidate, _dy)) ;

}

#endif /* HIDTOUCH__FISHNETLOCATOR__H */
