/****
HidTouch input driver.
(c)2008 David SPORN
 ****/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef HIDTOUCH__BODY__H
#define HIDTOUCH__BODY__H



/**** Device control event handler -- declaration ****/
static int
hdtOnDeviceInit(DeviceIntPtr device) ;

static int
hdtOnDeviceInit__initButtons(DeviceIntPtr device);

static int
hdtOnDeviceInit__initAxes(DeviceIntPtr device);

static void
hdtOnDeviceInit__initAxes__MinMax(DeviceIntPtr device, HdtDevicePtr pDevice, ScrnInfoPtr pScreen);

static void
hdtOnDeviceInit__initAxes__FourCorners(DeviceIntPtr device, HdtDevicePtr pDevice, ScrnInfoPtr pScreen);

static int
hdtOnDeviceOn(DeviceIntPtr device) ;

static int
hdtOnDeviceOff(DeviceIntPtr device) ;

static int
hdtOnDeviceClose(DeviceIntPtr device) ;

static int
hdtOnReadInput(InputInfoPtr pInfo) ;

static int
hdtOnReadInput__updatePosition(InputInfoPtr pInfo, HdtDevicePtr pDevice, HdtRawDataPtr rawData) ;

static void
hdtOnReadInput__updatePosition__MinMax(InputInfoPtr pInfo, HdtDevicePtr pDevice) ;

static void
hdtOnReadInput__updatePosition__FourCorners(InputInfoPtr pInfo, HdtDevicePtr pDevice) ;

static void
hdtOnReadInput__postMotionEvent(InputInfoPtr pInfo, int x, int y) ;

static void
hdtOnReadInput__postButtonEvent(InputInfoPtr pInfo, int x, int y, int leftButtonState) ;





/**** Device control event handler -- definition ****/
static int
hdtOnDeviceInit(DeviceIntPtr device)
{
    hdtOnDeviceInit__initButtons(device) ;
    hdtOnDeviceInit__initAxes(device) ;

    return Success;
}

static int
hdtOnDeviceOn(DeviceIntPtr device)
{
    InputInfoPtr pInfo = device->public.devicePrivate;
    HdtDevicePtr pDevice = pInfo->private;

    xf86Msg(X_INFO, "%s (%s): On.\n", pInfo->name, pDevice->myConfiguration.device);
    if (device->public.on)
    {
        xf86Msg(X_ERROR, "%s: device already open.\n", pInfo->name);
        return BadRequest;
    }

    SYSCALL(pInfo->fd = open(pDevice->myConfiguration.device, O_RDONLY | O_NONBLOCK));
    if (pInfo->fd < 0)
    {
        xf86Msg(X_ERROR, "%s: cannot open device.\n", pInfo->name);
        return BadRequest;
    }

    xf86FlushInput(pInfo->fd);
    xf86AddEnabledDevice(pInfo);
    device->public.on = TRUE;

    return Success;
}

static int
hdtOnDeviceOff(DeviceIntPtr device)
{
    InputInfoPtr pInfo = device->public.devicePrivate;
    HdtDevicePtr pDevice = pInfo->private;

    xf86Msg(X_INFO, "%s: Off.\n", pInfo->name);
    if (!device->public.on)
    {
        xf86Msg(X_ERROR, "%s: device already off.\n", pInfo->name);
        return BadRequest;
    }

    xf86RemoveEnabledDevice(pInfo);
    pInfo->fd = -1;
    device->public.on = FALSE;

    return Success;
}

static int
hdtOnDeviceClose(DeviceIntPtr device)
{
    return Success;
}


static int
hdtOnDeviceInit__initButtons(DeviceIntPtr device)
{
    InputInfoPtr        pInfo = device->public.devicePrivate;
    CARD8               *map;
    int                 i;
    const int           num_buttons = 3;
    int                 ret = Success;

    map = xcalloc(num_buttons, sizeof(CARD8));

    for (i = 0; i < num_buttons; i++)
    {
        map[i] = i;
    }

    if (!InitButtonClassDeviceStruct(device, num_buttons, map)) {
        xf86Msg(X_ERROR, "%s: Failed to register buttons.\n", pInfo->name);
        ret = BadAlloc;
    }

    xfree(map);
    return ret;
}

/******** Init axes section ********/

static int
hdtOnDeviceInit__initAxes(DeviceIntPtr device)
{
    InputInfoPtr pInfo = device->public.devicePrivate;
    HdtDevicePtr pDevice = pInfo->private;
    const int           num_axes = 2;


    if (!InitValuatorClassDeviceStruct(device,
            num_axes,
#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 3
            GetMotionHistory,
#endif
            GetMotionHistorySize(),
            Absolute))
    {
        return BadAlloc;
    }

    pInfo->dev->valuator->mode = Absolute;
    if (!InitAbsoluteClassDeviceStruct(device))
    {
        return BadAlloc;
    }

    ScrnInfoPtr   pScreen = xf86Screens[0];

    switch(pDevice->myConfiguration.calibration.model)
    {
    case HDT__CONFIGURATION_VALUE__CALIBRATION__MODEL__MIN_MAX:
        hdtOnDeviceInit__initAxes__MinMax(device, pDevice, pScreen);
    	break;
    case HDT__CONFIGURATION_VALUE__CALIBRATION__MODEL__FOUR_CORNER:
    	hdtOnDeviceInit__initAxes__FourCorners(device, pDevice, pScreen);
    	break;
    default:
    	//do nothing (silently fail)
    	xf86Msg(X_ERROR, "%s: no calibration model specified, will do nothing from now...\n", pInfo->name);
    	break;

    }

    return Success;
}

static void
hdtOnDeviceInit__initAxes__MinMax(DeviceIntPtr device, HdtDevicePtr pDevice, ScrnInfoPtr pScreen)
{
    xf86InitValuatorAxisStruct(device, 0,
            pDevice->myConfiguration.calibration.minX,
            pDevice->myConfiguration.calibration.maxX,
            pScreen->virtualX,
            0,
            pScreen->virtualX);
    xf86InitValuatorDefaults(device, 0);

    xf86InitValuatorAxisStruct(device, 1,
            pDevice->myConfiguration.calibration.minY,
            pDevice->myConfiguration.calibration.maxY,
            pScreen->virtualY,
            0,
            pScreen->virtualY);
    xf86InitValuatorDefaults(device, 1);

}

/*
 * In the "four corners" model, we compute screen coordinate
 * directly, thus we set up axis using
 * 0-->screenWidth and 0-->screenHeight.
 */
static void
hdtOnDeviceInit__initAxes__FourCorners(DeviceIntPtr device, HdtDevicePtr pDevice, ScrnInfoPtr pScreen)
{
    xf86InitValuatorAxisStruct(device, 0,
            0,
            pDevice->myConfiguration.calibration.screenWidth,
            pScreen->virtualX,
            0,
            pScreen->virtualX);
    xf86InitValuatorDefaults(device, 0);

    xf86InitValuatorAxisStruct(device, 1,
            0,
            pDevice->myConfiguration.calibration.screenHeight,
            pScreen->virtualY,
            0,
            pScreen->virtualY);
    xf86InitValuatorDefaults(device, 1);
}

/******** Read Input section ********/

static int
hdtOnReadInput(InputInfoPtr pInfo)
{
    HdtDevicePtr pDevice = pInfo->private;

    HdtRawDataRec raw_data ;
    raw_data.pressure = -1 ;
    raw_data.x = -1 ;
    raw_data.y = -1 ;
    HdtRawData__fillFromInputInfo(&raw_data, pInfo) ;

    int has_changed = 0 ;
    hdtOnReadInput__updatePosition(pInfo, pDevice, &raw_data) ;

    if (0 != has_changed)
    {
        pDevice->myRawData.timestamp = GetTimeInMillis();
    }

    return Success ;
}

/*
 * Translate raw data into axis coordinate
 * and post event as needed
 * return >0 if raw data has changed
 */
static int
hdtOnReadInput__updatePosition(InputInfoPtr pInfo, HdtDevicePtr pDevice, HdtRawDataPtr rawData)
{
    int has_changed = 0 ;
    if ((-1 < rawData->x && rawData->x != pDevice->myRawData.x)
            || (-1 < rawData->y && rawData->y != pDevice->myRawData.y))
    {
        has_changed += 1 ;
        pDevice->myRawData.x = rawData->x ;
        pDevice->myRawData.y = rawData->y ;

        switch(pDevice->myConfiguration.calibration.model)
        {
        case HDT__CONFIGURATION_VALUE__CALIBRATION__MODEL__MIN_MAX:
        	hdtOnReadInput__updatePosition__MinMax(pInfo, pDevice);
        	break;
        case HDT__CONFIGURATION_VALUE__CALIBRATION__MODEL__FOUR_CORNER:
        	hdtOnReadInput__updatePosition__FourCorners(pInfo, pDevice);
        	break;
        default:
        	break;
        }

        hdtOnReadInput__postMotionEvent(pInfo
        		, pDevice->myTranslatedData.x
        		, pDevice->myTranslatedData.y);

    }
    if (-1 < rawData->pressure && rawData->pressure != pDevice->myRawData.pressure)
    {
        has_changed += 2 ;
        pDevice->myRawData.pressure = rawData->pressure ;
        /*no need for a special translation process*/
        pDevice->myTranslatedData.pressure = pDevice->myRawData.pressure ;
        hdtOnReadInput__postButtonEvent(pInfo
        		, pDevice->myTranslatedData.x
        		, pDevice->myTranslatedData.y
        		, pDevice->myTranslatedData.pressure);
    }

    return has_changed ;
}

static void
hdtOnReadInput__updatePosition__MinMax(InputInfoPtr pInfo, HdtDevicePtr pDevice)
{
    /*no need for a special translation process*/
    pDevice->myTranslatedData.x = pDevice->myRawData.x ;
    pDevice->myTranslatedData.y = pDevice->myRawData.y ;
}

static void
hdtOnReadInput__updatePosition__FourCorners(InputInfoPtr pInfo, HdtDevicePtr pDevice)
{
    /*Use fishnet locator*/
    pDevice->myTranslatedData.x =
    	HdtFishNetLocator__locateHorizontalPosition(&(pDevice->myLocator)
    			,FixedPoint8__new(pDevice->myRawData.x)
    			,FixedPoint8__new(pDevice->myRawData.y)
    			,HDT__FISH_NET_LOCATOR__STEP_COUNT_FOR_LOCATE) ;
    pDevice->myTranslatedData.y =
    HdtFishNetLocator__locateVerticalPosition(&(pDevice->myLocator)
			,FixedPoint8__new(pDevice->myRawData.x)
			,FixedPoint8__new(pDevice->myRawData.y)
			,HDT__FISH_NET_LOCATOR__STEP_COUNT_FOR_LOCATE) ;
    /*
    xf86Msg(X_INFO, "Position : %i, %i.\n",
                    pDevice->myTranslatedData.x, pDevice->myTranslatedData.y);
    */

}

static void
hdtOnReadInput__supportXrandr(InputInfoPtr pInfo, int* pX, int* pY)
{
    HdtDevicePtr pDevice = pInfo->private;
    HdtConfigurationRec configuration = pDevice->myConfiguration ;

    /* skip if min_max model */
    if (HDT__CONFIGURATION_VALUE__CALIBRATION__MODEL__MIN_MAX == configuration.calibration.model)
    {
    	return ;
    }

	ScrnInfoPtr   pScreenInfo = xf86Screens[0];
	ScreenPtr pScreen = pScreenInfo->pScreen ;
	Rotation rotation = xf86GetRotation(pScreen) ;

	int _hard_width = configuration.calibration.screenWidth ;
	int _hard_height = configuration.calibration.screenHeight ;
	int x = (*pX) ;
	int y = (*pY) ;

	/*
    xf86Msg(X_INFO, "Rotation : %i among %i, %i, %i, %i.\n",
    		rotation, RR_Rotate_0, RR_Rotate_90, RR_Rotate_180, RR_Rotate_270);
	*//*
	 * TODO: find how to get the current rotation !
	 */
	switch(rotation)
	{
	case RR_Rotate_0:
		break;
	case RR_Rotate_90:
	{
		fp8 tempy = FixedPoint8__new(_hard_width - x) ;
		tempy = FixedPoint8__div(tempy, FixedPoint8__new(_hard_width)) ;
		tempy *= (fp8) _hard_height ;
		fp8 tempx = FixedPoint8__new(y) ;
		tempx = FixedPoint8__div(tempx, FixedPoint8__new(_hard_height)) ;
		tempx *= (fp8) _hard_width ;
		*pX = FixedPoint8__toInt(tempx) ;
		*pY = FixedPoint8__toInt(tempy) ;
	}
		break;
	case RR_Rotate_180:
	{
		*pX = _hard_width - x ;
		*pY = _hard_height - y ;
	}
		break;
	case RR_Rotate_270:
	{
		fp8 tempy = FixedPoint8__new(x) ;
		tempy = FixedPoint8__div(tempy, FixedPoint8__new(_hard_width)) ;
		tempy *= (fp8) _hard_height ;
		fp8 tempx = FixedPoint8__new(_hard_height - y) ;
		tempx = FixedPoint8__div(tempx, FixedPoint8__new(_hard_height)) ;
		tempx *= (fp8) _hard_width ;
		*pX = FixedPoint8__toInt(tempx) ;
		*pY = FixedPoint8__toInt(tempy) ;
	}
		break;
	default:
		/*do nothing*/
		break;
	}
}

static void
hdtOnReadInput__postMotionEvent(InputInfoPtr pInfo, int x, int y)
{
	int trans_x = x ;
	int trans_y = y ;
	hdtOnReadInput__supportXrandr(pInfo, &trans_x, &trans_y);

    /* /
    xf86Msg(X_INFO, "Position : %i, %i ==> %i, %i.\n",
                    x, y, trans_x, trans_y);
    /* */
	xf86PostMotionEvent(pInfo->dev, 1
            , 0, 2
            , trans_x, trans_y);
}

static void
hdtOnReadInput__postButtonEvent(InputInfoPtr pInfo, int x, int y, int leftButtonState)
{
	int trans_x = x ;
	int trans_y = y ;
	hdtOnReadInput__supportXrandr(pInfo, &trans_x, &trans_y);

    xf86PostButtonEvent(pInfo->dev, 1, HDT__BUTTON_EVENT__LEFT_BUTTON
            ,leftButtonState ,0, 2
            ,trans_x, trans_y) ;

}



#endif /** ifndef HIDTOUCH__BODY__H **/
