/****
HidTouch input driver.
(c)2008 David SPORN

Code written after the random driver tutorial
http://www.x.org/wiki/Development/Documentation/XorgInputHOWTO
 ****/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef HIDTOUCH__XORG_DRIVER__DRIVER__H
#define HIDTOUCH__XORG_DRIVER__DRIVER__H



/**** Driver part -- declaration ****/

/* Life cycle */
static InputInfoPtr
hdtPreInit(InputDriverPtr drv, IDevPtr dev, int flags) ;

static void
hdtUnInit(InputDriverPtr drv, InputInfoPtr pInfo, int flags) ;

/* Operation */
static void
hdtReadInput(InputInfoPtr pInfo) ;

static int
hdtControl(DeviceIntPtr device, int what) ;

_X_EXPORT InputDriverRec HDT = {
    1,
    "HidTouch",
    NULL,
    hdtPreInit,
    hdtUnInit,
    NULL,
    0
};

/**** Driver part -- definition ****/

/**
 * Called each time a new device is to be initialized.
 * Init structs, sockets, whatever else needed to do.
 **/
static InputInfoPtr
hdtPreInit(InputDriverPtr drv, IDevPtr dev, int flags)
{
    InputInfoPtr pInfo;
    HdtDevicePtr pDevice;

    if (!(pInfo = xf86AllocateInput(drv, 0)))
        return NULL;

    pDevice = xcalloc(1, sizeof(HdtDeviceRec));
    if (!pDevice)
    {
        pInfo->private = NULL;
        xf86DeleteInput(pInfo, 0);
        return NULL;
    }

    pInfo->private      = pDevice;

    pInfo->name         = xstrdup(dev->identifier);
    pInfo->flags        = 0;
    pInfo->type_name    = XI_MOUSE; /* see XI.h */
    pInfo->conf_idev    = dev;
    pInfo->read_input   = hdtReadInput; /* new data avl */
    pInfo->switch_mode  = NULL;            /* toggle absolute/relative mode */
    pInfo->device_control = hdtControl; /* enable/disable dev */


    /* process driver specific options */
    HdtConfiguration__retrieve(&(pDevice->myConfiguration), dev) ;

    /* process generic options */
    xf86CollectInputOptions(pInfo, NULL, NULL);
    xf86ProcessCommonOptions(pInfo, pInfo->options);


    /* Open sockets, init device files, etc. */
    SYSCALL(pInfo->fd = open(pDevice->myConfiguration.device, O_RDWR | O_NONBLOCK));
    if (pInfo->fd == -1)
    {
        xf86Msg(X_ERROR, "%s: failed to open %s.\n",
                pInfo->name, pDevice->myConfiguration.device);
        pInfo->private = NULL;
        xfree(pDevice);
        xf86DeleteInput(pInfo, 0);
        return NULL;
    }
    /* do more funky stuff */

    if (HDT__CONFIGURATION_VALUE__CALIBRATION__MODEL__FOUR_CORNER == pDevice->myConfiguration.calibration.model)
    {
        xf86Msg(X_ERROR, "%s: start four corners model initialization.\n",
                pInfo->name);
        HdtCalibrationRec _calibration = pDevice->myConfiguration.calibration ;
    	int width = _calibration.screenWidth ;
    	int height = _calibration.screenHeight ;
    	HdtFishNetLocatorPtr _fishnet = &(pDevice->myLocator) ;
    	if (!(HdtFishNetLocator__instanciate(_fishnet, width, height)))
    	{
            xf86Msg(X_ERROR, "%s: failed to allocate enough memory.\n",
                    pInfo->name);
            pInfo->private = NULL;
            xfree(pDevice);
            xf86DeleteInput(pInfo, 0);
            return NULL;
    	}
    	HdtFishNetLocator__init(_fishnet
    			,_calibration.cornerTopLeftX, _calibration.cornerTopLeftY
    			,_calibration.cornerTopRightX, _calibration.cornerTopRightY
    			,_calibration.cornerBottomLeftX, _calibration.cornerBottomLeftY
    			,_calibration.cornerBottomRightX, _calibration.cornerBottomRightY
    			) ;
        xf86Msg(X_ERROR, "%s: done four corners model initialization.\n",
                pInfo->name);
    }


    /* close file again */
    close (pInfo->fd);
    pInfo->fd = -1;

    /* set the required flags */
    pInfo->flags |= XI86_OPEN_ON_INIT;
    pInfo->flags |= XI86_CONFIGURED;

    return pInfo;
}

static void
hdtUnInit(InputDriverPtr drv, InputInfoPtr pInfo, int flags)
{
    HdtDevicePtr pDevice = pInfo->private;

    if (pDevice->myConfiguration.device)
    {
        xfree(pDevice->myConfiguration.device);
        pDevice->myConfiguration.device = NULL;
    }

    HdtFishNetLocatorPtr _fishnet = &(pDevice->myLocator) ;
    HdtFishNetLocator__destroy(_fishnet) ;

    xfree(pDevice);
    xf86DeleteInput(pInfo, 0);
}

static void
hdtReadInput(InputInfoPtr pInfo)
{
    /*while(xf86WaitForInput(pInfo->fd, 0) > 0)*/
    {
        hdtOnReadInput(pInfo);
    }
}

static int
hdtControl(DeviceIntPtr device, int what)
{
    /*InputInfoPtr pInfo = device->public.devicePrivate;
    HdtDevicePtr pDevice = pInfo->private;*/

    switch(what)
    {
        case DEVICE_INIT:
            return hdtOnDeviceInit(device) ;
            break;
        /* Switch device on. Establish socket, start event delivery.  */
        case DEVICE_ON:
            return hdtOnDeviceOn(device) ;
            break;
        /**
         * Shut down device.
         */
        case DEVICE_OFF:
            return hdtOnDeviceOff(device) ;
            break;
        case DEVICE_CLOSE:
            return hdtOnDeviceClose(device) ;
            /* free what we have to free */
            break;
    }
    return Success;
}



#endif /** ifndef HIDTOUCH__XORG_DRIVER__DRIVER__H **/
